#[derive(Debug, StructOpt)]
#[structopt(
name = "awatch",
about = "Here to help you with watching anime on your PC."
)]
pub struct Opt {
    #[structopt(subcommand)]
    pub mode: Option<RunMode>
}

#[derive(StructOpt, Debug)]
pub enum ShellType {
    #[structopt(
    about = "Generates a .bash completion file for the Bourne Again SHell (BASH)",
    name = "bash"
    )]
    Bash,
    #[structopt(
    about = "Generates a .fish completion file for the Friendly Interactive SHell (fish)",
    name = "fish"
    )]
    Fish,
    #[structopt(
    about = "Generates a completion file for the Z SHell (ZSH)",
    name = "zsh"
    )]
    Zsh,
    #[structopt(
    about = "Generates a completion file for PowerShell",
    name = "ps"
    )]
    PowerShell,
    #[structopt(
    about = "Generates a completion file for Elvish",
    name = "elvish"
    )]
    Elvish,
}

#[derive(StructOpt, Debug)]
pub enum RunMode {
    #[structopt(name = "init", about = "Initialize config")]
    Init,
    #[structopt(name = "play", about = "Play next episode")]
    Play,
    #[structopt(name = "prev", about = "Decrease episode counter by one")]
    Prev,
    #[structopt(name = "next", about = "Increase episode counter by one")]
    Next,
    #[structopt(name = "update", about = "Update saved config")]
    Update,
    #[structopt(name = "reset", about = "Set episode to 0")]
    Reset,
    #[structopt(name = "completion", about = "Generate autocompletion \
     for your shell. If no shell was specified, then it will try \
     to recognize it automatically.")]
    Completion {
        #[structopt(subcommand)]
        shell: Option<ShellType>
    },
}

impl From<ShellType> for Shell {
    fn from(shell: ShellType) -> Self {
        match shell {
            ShellType::Bash => { Shell::Bash }
            ShellType::Fish => { Shell::Fish }
            ShellType::Zsh => { Shell::Zsh }
            ShellType::PowerShell => { Shell::PowerShell }
            ShellType::Elvish => { Shell::Elvish }
        }
    }
}

fn recognize_shell() -> AppResult<Shell> {
    println!("Started auto shell recognition.");
    let shell_var = std::env::var("SHELL").map_err(|_| {
        AppError::RuntimeError(String::from(
            "$SHELL env variable not found please specify shell by yourself",
        ))
    })?;
    let shell_split: Vec<_> = shell_var.split('/').collect();
    if shell_split.is_empty() {
        return Err(AppError::RuntimeError(String::from(
            "Can't recognize shell please specify it by yourself.",
        )));
    }
    let shell_name = shell_split.last().unwrap();
    Shell::from_str(shell_name).map_err(|_| AppError::RuntimeError(String::from("Unknown shell")))
}

pub fn generate_completion(shell: Option<ShellType>) -> AppResult<()> {
    let shell = if let Some(shell_val) = shell {
        Shell::from(shell_val)
    } else {
        let current_shell = recognize_shell()?;
        println!("Recognized shell: {}", current_shell.to_string());
        current_shell
    };
    let home_dir = dirs::home_dir();
    let mut target_dir = String::from(".");
    if let Some(home_dir) = home_dir {
        target_dir = match shell {
            Shell::Bash => {
                String::from("/etc/bash_completion.d")
            }
            Shell::Fish => {
                let fish_dir = format!("{}/.config/fish/completions/", home_dir.display());
                fish_dir
            }
            Shell::Zsh => {
                let zsh_dir = std::env::var("ZSH")
                    .map_err(|_| AppError::RuntimeError(String::from(
                        "Please install oh-my-zsh for rich autocompletions."
                    )))?;
                format!("{}/completions", zsh_dir)
            }
            Shell::PowerShell => { String::from(".") }
            Shell::Elvish => { String::from(".") }
        };
    }
    std::fs::create_dir_all(target_dir.as_str()).ok();
    Opt::clap().gen_completions(env!("CARGO_PKG_NAME"), shell, target_dir.as_str());
    println!("Completion file saved at {}", target_dir);
    Ok(())
}